import pygame, math, sys, random
from pygame.locals import *
pygame.init()

window = pygame.display.set_mode((700,500))

framerate = 40.0 #frames per second

bomb_origin_x = 350
bomb_origin_y = 386

groundlevel = 465

pygame.mouse.set_visible(False)
pygame.mixer.music.load("data/music.mp3")

music_playing = True

class bomb:
	def __init__(self,x,y,velocity,angle,direction):	#velocity in meters per second
		self.xi=x
		self.yi=y
		self.angle = angle
		self.direction = direction
		self.vy=velocity*(10/framerate)*math.sin(angle*math.pi/180.0)	#velocity in pixels per frame
		self.vx=velocity*(10/framerate)*math.cos(angle*math.pi/180.0)	#velocity in pixels per frame
		self.image = pygame.image.load("data/bomb.png")
		self.image.set_colorkey((255,255,255))
		self.tick = 0 #frames
		self.exploded = False

	def get_x(self):
		return int(self.xi + self.vx*self.tick*self.direction)	
			
	def get_y(self):
		return int(self.yi - self.vy*self.tick + 0.5*(9.8/framerate)*self.tick**2)
		
	def get_explode_x(self, groundlevel):
		t =(self.vy + math.sqrt(self.vy**2-(19.6/framerate)*(self.yi-groundlevel)))/(9.8/framerate)
		
		return int(self.xi + self.vx*t*self.direction)	
		
	def update(self):
		self.tick += 1
		y = self.get_y()
		if y < groundlevel:
			window.blit(self.image,((self.get_x(),y)))
		else:
			self.exploded = True
			x = self.get_explode_x(groundlevel)
			explosions.append(explosion(x,groundlevel))	
			for z in zombies:
				if abs(z.pos.centerx - x) <= 15:
					z.dead = True
			
class zombie:
	def __init__(self):
		self.images=[]
	
		self.dead = False
		
		if random.randint(0,1) == 1:
			self.direction = -1
			self.pos=pygame.Rect(700,groundlevel-25,13,25)
			for i in range(4):
				self.images.append(pygame.image.load("data/z"+str(i)+".png"))
			for i in range(13):
				self.images.append(pygame.image.load("data/d"+str(i)+".png"))
		else:
			self.direction = 1
			self.pos=pygame.Rect(-25,groundlevel-25,13,25)
			for i in range(4):
				self.images.append(pygame.transform.flip(pygame.image.load("data/z"+str(i)+".png"),True,False))
			for i in range(13):
				self.images.append(pygame.transform.flip(pygame.image.load("data/d"+str(i)+".png"),True,False))
		
		for i in self.images:
			i.set_colorkey((255,255,255))		
		
		self.tick = 0
		self.frame = 0
		
	def update(self):
		if not self.dead:
			self.tick+=1
			self.tick = self.tick%10
			
			if self.tick == 0:
				self.frame += 1
				self.frame = self.frame%4
				if self.frame%2 == 0:
					self.pos[0] += self.direction
		
		else:
			if self.frame < 3:
				self.frame = 3
		
			self.tick+=1
			self.tick = self.tick%2
			
			if self.tick == 0:
				self.frame += 1			
								
		window.blit(self.images[self.frame],self.pos)
		
		if self.pos.centerx < 370 and self.pos.centerx > 230:
			game_over()	
		
class explosion:
	def __init__(self,x,y):
		self.rect = pygame.Rect(0,0,10,10)
		self.rect.midbottom =(x,y)
		self.images = []
		for i in range(4):
			self.images.append(pygame.image.load("data/e"+str(i)+".png"))
		for i in self.images:
			i.set_colorkey((255,255,255))
		self.frame = 0
		
	def update(self):
		window.blit(self.images[self.frame],self.rect)	
		self.frame += 1

def quit():
	f = file("data/highscore.hsc",'w')
	f.write(str(highscore))
	f.close()
	pygame.quit()
	sys.exit()		

def intro():
	pygame.mixer.music.play(-1)

	background = pygame.image.load("data/background.png")
	window.blit(background, ((0,0)))
	
	font = pygame.font.SysFont(None,20) 
	
	message = "The world was destroyed in a nuclear war. Cities were laid to waste and many of those who survived were mutated into flesh-eating zombies. The unfortunate remainder of humanity has been left to deal with the aftermath. Luckily Bob has a tank full of lighter fluid to make Molotov cocktails with and Alice has a good throwing arm. Even so, they'll be lucky to survive the assault on their house... (press any key to start)"
	
	messages = message.split(" ")
	
	line = ""
	
	line_num = 0
	
	for i in messages:
		if len(line) > 0:
			line2 = line + " " + i
		else:
			line2 = line + i
		message = font.render(line2,True,(0,0,0),(255,255,255))
		if message.get_width() < 660:
			line = line2
		else:
			message = font.render(line,True,(0,0,0),(255,255,255))
			line = i
			window.blit(message,(20,120+line_num*20))
			line_num += 1
	
	message = font.render(line,True,(0,0,0),(255,255,255))
	window.blit(message,(20,120+line_num*20))		
			
	pygame.display.update()
	
	while True:
		for event in pygame.event.get():
			if event.type == QUIT:
				return
	
			if event.type == MOUSEBUTTONUP:
				return
					
			if event.type == KEYUP:	
				return	
	
def game_over():
	background = pygame.image.load("data/background.png")
	window.blit(background, ((0,0)))
	
	font = pygame.font.SysFont(None,20) 
	large_font = pygame.font.SysFont(None,50)
	
	message = large_font.render("GAME OVER",True,(0,0,0),(255,255,255))
	width = message.get_width()
	height = message.get_height()	
	window.blit(message,((350-width/2,250-height)))
	message = font.render("Alice and Bob were overrun by zombies and eaten.",True,(0,0,0),(255,255,255))
	width = message.get_width()	
	window.blit(message,((350-width/2,300)))
	
	score_image = font.render("Score:  "+str(score),True,(0,0,0),(255,255,255))
	width = score_image.get_width()	
	window.blit(score_image,((680-width,20)))
	score_image = font.render("High Score:  "+str(highscore),True,(0,0,0),(255,255,255))
	width = score_image.get_width()	
	window.blit(score_image,((680-width,40)))
	
	pygame.display.update()
	
	while True:
		for event in pygame.event.get():
			if event.type == QUIT:
				quit()
	
			if event.type == MOUSEBUTTONUP:
				quit()
					
			if event.type == KEYUP:					
				quit()	
					
def run():
	global explosions, zombies, highscore, score, music_playing

	#set up
	
	background = pygame.image.load("data/background.png")
	slider = pygame.image.load("data/pointer.png")
	ruler = pygame.image.load("data/ruler.png")
	cursor = pygame.image.load("data/cursor.png")
	cursor.set_colorkey((255,255,255))
	
	clock=pygame.time.Clock()
	
	score = 0
	
	try:
		f = file("data/highscore.hsc",'r')
		highscore = int(f.read())
		f.close()
	except:
		#print "highscore not found"
		highscore = 0
	
	num_bombs = 7
	bombs_on_order = 0
	
	bombs = []
	zombies = []
	explosions = []
	
	alice = []
	for i in range(4):
		alice.append(pygame.image.load("data/a"+str(i)+".png"))
	for i in range(13):
		alice.append(pygame.image.load("data/aa"+str(i)+".png"))
		
	for i in alice:
		i.set_colorkey((255,255,255))
	
	bob = []
	for i in range(19):
		bob.append(pygame.image.load("data/b"+str(i)+".png"))	
		
	for i in bob:
		i.set_colorkey((255,255,255))
		
	font = pygame.font.SysFont(None,20)
	
	box_image = pygame.image.load("data/box.png")
	box_rect = pygame.Rect(bomb_origin_x-19,bomb_origin_y+17,13,9)
	
		#controls
	mouse_down = False
	up_down = False
	down_down = False
	left_down = False
	right_down = False
	
	v_slider_clicked = False
	a_slider_clicked = False
	
	velocity = 0
	angle = 0
	direction = 1	
	
	throwcount = 16
	makecount = 6
	
	#loop
	
	while True:
		
		#update
		
			#add zombies
		if random.randint(0,300) == 1:
			zombies.append(zombie())
			bombs_on_order += 1
				
			#throw bomb
		if throwcount == 4:
			bombs.append(bomb(bomb_origin_x,bomb_origin_y,velocity,angle,direction))
			num_bombs -= 1
			
		if makecount == 4:
			bombs.append(bomb(bomb_origin_x-100,bomb_origin_y+50,23,65,1))
			bombs_on_order -= 1
			
		window.blit(background, ((0,0)))
		
			#alice
			
		window.blit(alice[throwcount],((bomb_origin_x-11,bomb_origin_y-1)))	
		
			#bob
			
		window.blit(bob[makecount],((bomb_origin_x-117,bomb_origin_y+32)))
		
		removed = 0	
		for i in range(len(zombies)):
			if i < len(zombies)-removed:
				zombies[i].update()
				if zombies[i].frame > 15:
					zombies = zombies[:i]+zombies[i+1:]	
					score += 10
					if score > highscore:
						highscore = score
					removed+=1	
		
		removed = 0	
		for i in range(len(bombs)):
			if i < len(bombs)-removed:
				bombs[i].update()
				if bombs[i].exploded:
					bombs = bombs[:i]+bombs[i+1:]	
					removed+=1
					
				elif bombs[i].get_y() < box_rect[1] and bombs[i].get_x() > box_rect[0]:
					x = bombs[i].get_explode_x(box_rect[1])
					if x > box_rect[0] and x < box_rect[0]+box_rect[3]:
						num_bombs += 1
						bombs = bombs[:i]+bombs[i+1:]	
						removed+=1 				
						
		removed = 0	
		for i in range(len(explosions)):
			if i < len(explosions)-removed:
				explosions[i].update()
				if explosions[i].frame > 3:
					explosions = explosions[:i]+explosions[i+1:]	
					removed+=1
			
		window.blit(box_image,box_rect)			
		
			#controls
	
		velocity = int(math.sqrt((pygame.mouse.get_pos()[0]-bomb_origin_x*1.0)**2+(pygame.mouse.get_pos()[1]-bomb_origin_y*1.0)**2)/3)-5
		
		if velocity < 0:
			velocity = 0
			
		if velocity > 50:
			velocity = 50			
		
		direction = 1;	
			
		if pygame.mouse.get_pos()[0] == bomb_origin_x and pygame.mouse.get_pos()[1] <= bomb_origin_y:
			angle = 90	
		
		elif pygame.mouse.get_pos()[0] == bomb_origin_x and pygame.mouse.get_pos()[1] > bomb_origin_y:
			angle = -90		
					
		else:	
		
			angle = int(math.atan(-(pygame.mouse.get_pos()[1]-bomb_origin_y*1.0)/(pygame.mouse.get_pos()[0]-bomb_origin_x*1.0))*180/math.pi)
			
			if pygame.mouse.get_pos()[0] < bomb_origin_x:
				angle = -angle
				direction = -1
				
			#if angle < 0:
			#	angle = 0
				
			if angle > 90:
				angle = 90		
		
			#aim
			
		x_begin = bomb_origin_x + 10*math.cos(angle*math.pi/180.0)*direction
		x_end = bomb_origin_x + (10+velocity)*math.cos(angle*math.pi/180.0)*direction
		
		y_begin = bomb_origin_y - 10*math.sin(angle*math.pi/180.0)
		y_end = bomb_origin_y - (10+velocity)*math.sin(angle*math.pi/180.0)
		
		pygame.draw.line(window,(0,0,0),(x_begin,y_begin),(x_end,y_end), 1)
		"""
		x2 = x_end + 3*math.cos((angle-135)*math.pi/180.0)*direction

		y2 = bomb_origin_y - 3*math.sin((angle-135)*math.pi/180.0)
		
		pygame.draw.line(window,(0,0,0),(x_end,y_end),(x2,y2), 1)
		
		x2 = x_end + 3*math.cos((angle+135)*math.pi/180.0)*direction

		y2 = bomb_origin_y - 3*math.sin((angle+135)*math.pi/180.0)
		
		pygame.draw.line(window,(0,0,0),(x_end,y_end),(x2,y2), 1)
		"""	
			#draw text
			
		window.blit(ruler,((0,470)))	
			
		window.blit(font.render(str(velocity)+" m/s",True,(0,0,0),(255,255,255)),((24,480)))
		window.blit(font.render(str(angle)+chr(248),True,(0,0,0),(255,255,255)),((159,480)))
		score_image = font.render("Score:  "+str(score),True,(0,0,0),(255,255,255))
		width = score_image.get_width()	
		window.blit(score_image,((680-width,20)))
		score_image = font.render("High Score:  "+str(highscore),True,(0,0,0),(255,255,255))
		width = score_image.get_width()	
		window.blit(score_image,((680-width,40)))
		window.blit(font.render("Molotov cocktails:  "+str(num_bombs),True,(0,0,0),(255,255,255)),((20,20)))	
			
		window.blit(slider,((24+velocity,473)))
		window.blit(slider,((251+angle,473)))
		
		#cursor_rect = pygame.Rect(pygame.mouse.get_pos()[0],pygame.mouse.get_pos()[1],15,15)
		
		window.blit(cursor,(pygame.mouse.get_pos()[0]-7,pygame.mouse.get_pos()[1]-7))
				
			#main
		
		if throwcount != 16 and not (num_bombs == 0 and throwcount == 8):
			throwcount += 1	
			
		if makecount != 6 or bombs_on_order > 0:
			makecount += 1
			makecount = makecount % 19
			
		clock.tick(int(framerate))	
			
		#draw
		
		pygame.display.update()
		
		#check for event
		
		for event in pygame.event.get():
			if event.type == QUIT:
				quit()
	
			if event.type == MOUSEBUTTONUP:
				mouse_down = False
				if throwcount == 16:
					throwcount = 0	
					
			if event.type == KEYUP:					
				if event.key == K_ESCAPE:
					quit()
					
				if event.key == K_m:
					music_playing = not music_playing
					
					if music_playing:
						pygame.mixer.music.play(-1)
					else:
						pygame.mixer.music.stop()

intro()				
run()	